'use strict';

var _require = require('./emitter'),
    createChangesFunnelEmitter = _require.createChangesFunnelEmitter;

var createTransactions = function createTransactions(models) {
  return models.map(function (model) {
    return model.createTransaction();
  });
};
var commitTransactions = function commitTransactions(models) {
  return models.map(function (model) {
    return model.commitTransaction();
  });
};
var abortTransactions = function abortTransactions(models) {
  return models.map(function (model) {
    return model.abortTransaction();
  });
};
var emitChanges = function emitChanges(models) {
  return createChangesFunnelEmitter({ models: models }).emitChange();
};

var createAction = function createAction(handler) {
  var models = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  return function () {
    var res = null;

    createTransactions(models);

    try {
      res = handler.apply(undefined, arguments);
      commitTransactions(models);
      emitChanges(models);
    } catch (error) {
      abortTransactions(models);
      throw error;
    }

    return res;
  };
};

var runAsAction = function runAsAction(handler, models) {
  return createAction(handler, models)();
};

module.exports = {
  createAction: createAction,
  runAsAction: runAsAction
};
