'use strict';

var error = require('./error');

var hasTransaction = function hasTransaction(manager) {
  if (manager.transaction) return;

  throw new error.TransactionDoesNotExist({
    message: 'A transaction is needed to perform this action.'
  });
};

var hasNoTransaction = function hasNoTransaction(manager) {
  if (!manager.transaction) return;

  throw new error.TransactionAlreadyExists({
    message: 'You need to abort or commit the existing one before creating another'
  });
};

var entryExists = function entryExists(transaction, id) {
  if (transaction.values.has(id)) return;

  throw new error.LostId({
    message: 'Id lost. Impossible to sync the entry.'
  });
};

module.exports = {
  hasTransaction: hasTransaction,
  hasNoTransaction: hasNoTransaction,
  entryExists: entryExists
};
