'use strict';

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _require = require('fbemitter'),
    EventEmitter = _require.EventEmitter;

var ulid = require('ulid');

var CHANGE_EVENT = 'change';

var ChangesEmitter = function () {
  function ChangesEmitter() {
    (0, _classCallCheck3.default)(this, ChangesEmitter);

    this.emitter = new EventEmitter();
  }

  (0, _createClass3.default)(ChangesEmitter, [{
    key: 'emitChange',
    value: function emitChange() {
      var event = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      this.emitter.emit(CHANGE_EVENT, event);
    }
  }, {
    key: 'changed',
    value: function changed(listener) {
      return this.emitter.addListener(CHANGE_EVENT, listener);
    }
  }]);
  return ChangesEmitter;
}();

var FunnelSubscription = function () {
  function FunnelSubscription(_ref) {
    var funnel = _ref.funnel,
        subscription = _ref.subscription;
    (0, _classCallCheck3.default)(this, FunnelSubscription);

    this.id = ulid();
    this.funnel = funnel;
    this.subscription = subscription;
  }

  (0, _createClass3.default)(FunnelSubscription, [{
    key: 'remove',
    value: function remove() {
      if (!this.subscription || !this.funnel) return;

      this.subscription.remove();
      this.funnel.removeSubscription(this);
      this.subscription = null;
      this.funnel = null;
    }
  }]);
  return FunnelSubscription;
}();

var getEmitters = function getEmitters(_ref2) {
  var _ref2$models = _ref2.models,
      models = _ref2$models === undefined ? [] : _ref2$models,
      _ref2$emitters = _ref2.emitters,
      emitters = _ref2$emitters === undefined ? [] : _ref2$emitters;
  return models.map(function (_ref3) {
    var emitter = _ref3.emitter;
    return emitter;
  }).concat(emitters);
};

var ChangesFunnelEmitter = function () {
  function ChangesFunnelEmitter() {
    var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : { models: [], emitters: [] };
    (0, _classCallCheck3.default)(this, ChangesFunnelEmitter);

    this.mainSubscriptions = [];
    this.mainEmitter = new ChangesEmitter();
    this.emitters = getEmitters(opts);
    this.subscriptions = {};
  }

  (0, _createClass3.default)(ChangesFunnelEmitter, [{
    key: 'isSameEvent',
    value: function isSameEvent(_ref4) {
      var _ref4$eventId = _ref4.eventId,
          eventId = _ref4$eventId === undefined ? null : _ref4$eventId;

      return this.eventId && this.eventId === eventId;
    }
  }, {
    key: 'forwardChangeEvent',
    value: function forwardChangeEvent() {
      var event = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      if (this.isSameEvent(event)) return;

      this.eventId = event.eventId;
      this.mainEmitter.emitChange(event);
    }
  }, {
    key: 'createMainSubscriptions',
    value: function createMainSubscriptions() {
      var _this = this;

      this.mainSubscriptions = this.emitters.map(function (emitter) {
        return emitter.changed(function (event) {
          return _this.forwardChangeEvent(event);
        });
      });
    }
  }, {
    key: 'clearMainSubscriptions',
    value: function clearMainSubscriptions() {
      this.mainSubscriptions.forEach(function (subscription) {
        return subscription.remove();
      });
      this.mainSubscriptions = [];
    }
  }, {
    key: 'addSubscription',
    value: function addSubscription(listener) {
      var subscription = new FunnelSubscription({
        subscription: this.mainEmitter.changed(listener),
        funnel: this
      });

      this.subscriptions[subscription.id] = subscription;

      return subscription;
    }
  }, {
    key: 'removeSubscription',
    value: function removeSubscription(subscription) {
      delete this.subscriptions[subscription.id];
      if (!this.subscriptionsCount()) this.clearMainSubscriptions();
    }
  }, {
    key: 'subscriptionsCount',
    value: function subscriptionsCount() {
      return Object.keys(this.subscriptions).length;
    }
  }, {
    key: 'changed',
    value: function changed(listener) {
      if (!this.subscriptionsCount()) this.createMainSubscriptions();

      return this.addSubscription(listener);
    }
  }, {
    key: 'emitChange',
    value: function emitChange() {
      var event = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      var broadcast = Object.assign({ eventId: ulid() }, event);

      return this.emitters.map(function (emitter) {
        return emitter.emitChange(broadcast);
      });
    }
  }]);
  return ChangesFunnelEmitter;
}();

var createChangesEmitter = function createChangesEmitter() {
  return new ChangesEmitter();
};
var createChangesFunnelEmitter = function createChangesFunnelEmitter(opts) {
  return new ChangesFunnelEmitter(opts);
};

module.exports = {
  ChangesEmitter: ChangesEmitter,
  ChangesFunnelEmitter: ChangesFunnelEmitter,
  createChangesEmitter: createChangesEmitter,
  createChangesFunnelEmitter: createChangesFunnelEmitter
};
