'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Immutable = require('immutable');

var map = function map(init) {
  return Immutable.Map(init);
};
var extractMaps = function extractMaps(_ref) {
  var _ref$maps = _ref.maps,
      maps = _ref$maps === undefined ? { src: map(), dest: map() } : _ref$maps;
  return maps;
};

var ReversedMapping = function () {
  function ReversedMapping(_ref2) {
    var mapping = _ref2.mapping;

    _classCallCheck(this, ReversedMapping);

    this.mapping = mapping;
    this.reversed = mapping;
  }

  _createClass(ReversedMapping, [{
    key: 'attach',
    value: function attach(srcKey, destKey) {
      return this.mapping.attach(destKey, srcKey);
    }
  }, {
    key: 'detach',
    value: function detach(srcKey, destKey) {
      return this.mapping.detach(destKey, srcKey);
    }
  }, {
    key: 'get',
    value: function get(destKey) {
      return this.mapping.getFromDest(destKey);
    }
  }]);

  return ReversedMapping;
}();

var Mapping = function () {
  function Mapping() {
    var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, Mapping);

    this.maps = extractMaps(opts);
    this.reversed = new ReversedMapping({ mapping: this });
  }

  _createClass(Mapping, [{
    key: 'getFromSrc',
    value: function getFromSrc(srcKey) {
      return this.maps.src.get(srcKey);
    }
  }, {
    key: 'getFromDest',
    value: function getFromDest(destKey) {
      return this.maps.dest.get(destKey);
    }
  }, {
    key: 'get',
    value: function get(srcKey) {
      return this.maps.src.get(srcKey);
    }
  }]);

  return Mapping;
}();

var OneToOneMapping = function (_Mapping) {
  _inherits(OneToOneMapping, _Mapping);

  function OneToOneMapping() {
    _classCallCheck(this, OneToOneMapping);

    return _possibleConstructorReturn(this, (OneToOneMapping.__proto__ || Object.getPrototypeOf(OneToOneMapping)).apply(this, arguments));
  }

  _createClass(OneToOneMapping, [{
    key: 'attach',
    value: function attach(srcKey, destKey) {
      var lastPointedDestKey = this.maps.src.get(srcKey);
      var lastPointedSrcKey = this.maps.dest.get(destKey);

      this.maps.src = this.maps.src.remove(lastPointedSrcKey).set(srcKey, destKey);
      this.maps.dest = this.maps.dest.remove(lastPointedDestKey).set(destKey, srcKey);
    }
  }, {
    key: 'normalizedDetach',
    value: function normalizedDetach(srcKey, destKey) {
      this.maps.src = this.maps.src.remove(srcKey);
      this.maps.dest = this.maps.dest.remove(destKey);
    }
  }, {
    key: 'detach',
    value: function detach(srcKey, destKey) {
      this.normalizedDetach(srcKey || this.maps.dest.get(destKey), destKey || this.maps.src.get(srcKey));
    }
  }]);

  return OneToOneMapping;
}(Mapping);

var OneToManyMapping = function (_Mapping2) {
  _inherits(OneToManyMapping, _Mapping2);

  function OneToManyMapping() {
    _classCallCheck(this, OneToManyMapping);

    return _possibleConstructorReturn(this, (OneToManyMapping.__proto__ || Object.getPrototypeOf(OneToManyMapping)).apply(this, arguments));
  }

  _createClass(OneToManyMapping, [{
    key: 'attach',
    value: function attach(srcKey, destKey) {
      var lastPointedDestKey = this.maps.src.get(srcKey);

      if (!this.maps.dest.has(destKey)) {
        this.maps.dest = this.maps.dest.set(destKey, Immutable.Set());
      }

      this.maps.src = this.maps.src.set(srcKey, destKey);

      if (lastPointedDestKey) {
        this.maps.dest = this.maps.dest.set(lastPointedDestKey, this.maps.dest.get(lastPointedDestKey).remove(srcKey));
      }

      this.maps.dest = this.maps.dest.set(destKey, this.maps.dest.get(destKey).add(srcKey));
    }
  }, {
    key: 'normalizedDetach',
    value: function normalizedDetach(srcKey, destKey) {
      this.maps.dest = this.maps.dest.set(destKey, this.maps.dest.get(destKey).remove(srcKey));
      this.maps.src = this.maps.src.remove(srcKey);
    }
  }, {
    key: 'detach',
    value: function detach(srcKey, destKey) {
      this.normalizedDetach(srcKey, destKey || this.maps.src.get(srcKey));
    }
  }]);

  return OneToManyMapping;
}(Mapping);

var ManyToOneMapping = function (_ReversedMapping) {
  _inherits(ManyToOneMapping, _ReversedMapping);

  function ManyToOneMapping() {
    _classCallCheck(this, ManyToOneMapping);

    return _possibleConstructorReturn(this, (ManyToOneMapping.__proto__ || Object.getPrototypeOf(ManyToOneMapping)).call(this, { mapping: new OneToManyMapping() }));
  }

  return ManyToOneMapping;
}(ReversedMapping);

var ManyToManyMapping = function (_Mapping3) {
  _inherits(ManyToManyMapping, _Mapping3);

  function ManyToManyMapping() {
    _classCallCheck(this, ManyToManyMapping);

    return _possibleConstructorReturn(this, (ManyToManyMapping.__proto__ || Object.getPrototypeOf(ManyToManyMapping)).apply(this, arguments));
  }

  _createClass(ManyToManyMapping, [{
    key: 'attach',
    value: function attach(srcKey, destKey) {
      if (!this.maps.src.has(srcKey)) this.maps.src = this.maps.src.set(srcKey, Immutable.Set());
      if (!this.maps.dest.has(destKey)) this.maps.dest = this.maps.dest.set(destKey, Immutable.Set());

      this.maps.src = this.maps.src.set(srcKey, this.maps.src.get(srcKey).add(destKey));
      this.maps.dest = this.maps.dest.set(destKey, this.maps.dest.get(destKey).add(srcKey));
    }
  }, {
    key: 'detach',
    value: function detach(srcKey, destKey) {
      this.maps.src = this.maps.src.set(srcKey, this.maps.src.get(srcKey).remove(destKey));
      this.maps.dest = this.maps.dest.set(destKey, this.maps.dest.get(destKey).remove(srcKey));
    }
  }]);

  return ManyToManyMapping;
}(Mapping);

module.exports = {
  OneToOneMapping: OneToOneMapping,
  OneToManyMapping: OneToManyMapping,
  ManyToOneMapping: ManyToOneMapping,
  ManyToManyMapping: ManyToManyMapping
};
