'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ulid = require('ulid');
var Immutable = require('immutable');

var _require = require('./model'),
    createEntryMixin = _require.createEntryMixin,
    Entry = _require.Entry;

var _require2 = require('./queryset'),
    QuerySet = _require2.QuerySet;

var _require3 = require('./mapping'),
    OneToOneMapping = _require3.OneToOneMapping,
    OneToManyMapping = _require3.OneToManyMapping,
    ManyToOneMapping = _require3.ManyToOneMapping,
    ManyToManyMapping = _require3.ManyToManyMapping;

var normalize = function normalize(_ref) {
  var _ref$model = _ref.model,
      model = _ref$model === undefined ? null : _ref$model,
      _ref$hasMany = _ref.hasMany,
      hasMany = _ref$hasMany === undefined ? false : _ref$hasMany,
      _ref$relatedName = _ref.relatedName,
      relatedName = _ref$relatedName === undefined ? null : _ref$relatedName;
  return { model: model, hasMany: hasMany, relatedName: relatedName };
};

var parse = function parse(opts) {
  return normalize(opts.model ? opts : { model: opts });
};

var createOneAccessor = function createOneAccessor(_ref2) {
  var Base = _ref2.Base,
      name = _ref2.name,
      mapping = _ref2.mapping,
      model = _ref2.model;
  return function (_Base) {
    _inherits(_class, _Base);

    function _class() {
      _classCallCheck(this, _class);

      return _possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
    }

    _createClass(_class, [{
      key: name,
      get: function get() {
        var key = mapping.get(this.getKey());

        return key ? model.objects.get(key) : null;
      },
      set: function set(other) {
        return other ? mapping.attach(this.getKey(), other.getKey()) : mapping.detach(this.getKey());
      }
    }]);

    return _class;
  }(Base);
};

var RelationQuerySet = function (_QuerySet) {
  _inherits(RelationQuerySet, _QuerySet);

  function RelationQuerySet(opts) {
    _classCallCheck(this, RelationQuerySet);

    var _this2 = _possibleConstructorReturn(this, (RelationQuerySet.__proto__ || Object.getPrototypeOf(RelationQuerySet)).call(this, opts));

    _this2.entry = opts.entry;
    _this2.mapping = opts.mapping;
    return _this2;
  }

  _createClass(RelationQuerySet, [{
    key: 'addExisting',
    value: function addExisting(other) {
      this.mapping.attach(this.entry.getKey(), other.getKey());
      return other;
    }
  }, {
    key: 'createAndAdd',
    value: function createAndAdd() {
      var rawValues = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      return this.addExisting(this.manager.create(rawValues));
    }
  }, {
    key: 'addOne',
    value: function addOne(other) {
      return other instanceof Entry ? this.addExisting(other) : this.createAndAdd(other);
    }
  }, {
    key: 'addMany',
    value: function addMany(others) {
      var _this3 = this;

      return others.map(function (other) {
        return _this3.addOne(other);
      });
    }
  }, {
    key: 'add',
    value: function add(oneOrMany) {
      return Array.isArray(oneOrMany) ? this.addMany(oneOrMany) : this.addOne(oneOrMany);
    }
  }, {
    key: 'remove',
    value: function remove(other) {
      return this.mapping.detach(this.entry.getKey(), other.getKey());
    }
  }]);

  return RelationQuerySet;
}(QuerySet);

var createManyAccessor = function createManyAccessor(_ref3) {
  var Base = _ref3.Base,
      name = _ref3.name,
      mapping = _ref3.mapping,
      model = _ref3.model;
  return function (_Base2) {
    _inherits(_class2, _Base2);

    function _class2() {
      _classCallCheck(this, _class2);

      return _possibleConstructorReturn(this, (_class2.__proto__ || Object.getPrototypeOf(_class2)).apply(this, arguments));
    }

    _createClass(_class2, [{
      key: name,
      get: function get() {
        var objects = model.objects;
        var keyName = objects.setup.get('keyName');
        var keyset = mapping.get(this.getKey());

        return new RelationQuerySet({
          entry: this,
          mapping: mapping,
          manager: objects,
          values: keyset ? objects.values.filter(function (values) {
            return keyset.has(values.get(keyName));
          }) : [],
          QuerySetConstructor: objects.QuerySetConstructor
        });
      }
    }]);

    return _class2;
  }(Base);
};

var createRelation = function createRelation(_ref4) {
  var name = _ref4.name,
      relatedName = _ref4.relatedName,
      relatedModel = _ref4.relatedModel,
      mapping = _ref4.mapping,
      accessorFactory = _ref4.accessorFactory,
      relatedAccessorFactory = _ref4.relatedAccessorFactory;
  return createEntryMixin(function (_ref5) {
    var Base = _ref5.Base,
        model = _ref5.model;

    if (relatedName) {
      relatedModel.inject(createEntryMixin(function (_ref6) {
        var RelatedBase = _ref6.Base;
        return relatedAccessorFactory({
          Base: RelatedBase,
          name: relatedName,
          mapping: mapping.reversed,
          model: model
        });
      }));
    }

    return accessorFactory({ Base: Base, name: name, mapping: mapping, model: relatedModel });
  });
};

var createOneToOneRelation = function createOneToOneRelation(_ref7) {
  var name = _ref7.name,
      relatedName = _ref7.relatedName,
      relatedModel = _ref7.relatedModel;
  return createRelation({
    name: name,
    relatedName: relatedName,
    relatedModel: relatedModel,
    mapping: new OneToOneMapping(),
    accessorFactory: createOneAccessor,
    relatedAccessorFactory: createOneAccessor
  });
};

var createOneToManyRelation = function createOneToManyRelation(_ref8) {
  var name = _ref8.name,
      relatedName = _ref8.relatedName,
      relatedModel = _ref8.relatedModel;
  return createRelation({
    name: name,
    relatedName: relatedName,
    relatedModel: relatedModel,
    mapping: new OneToManyMapping(),
    accessorFactory: createOneAccessor,
    relatedAccessorFactory: createManyAccessor
  });
};

var createManyToOneRelation = function createManyToOneRelation(_ref9) {
  var name = _ref9.name,
      relatedName = _ref9.relatedName,
      relatedModel = _ref9.relatedModel;
  return createRelation({
    name: name,
    relatedName: relatedName,
    relatedModel: relatedModel,
    mapping: new ManyToOneMapping(),
    accessorFactory: createManyAccessor,
    relatedAccessorFactory: createOneAccessor
  });
};

var createManyToManyRelation = function createManyToManyRelation(_ref10) {
  var name = _ref10.name,
      relatedName = _ref10.relatedName,
      relatedModel = _ref10.relatedModel;
  return createRelation({
    name: name,
    relatedName: relatedName,
    relatedModel: relatedModel,
    mapping: new ManyToManyMapping(),
    accessorFactory: createManyAccessor,
    relatedAccessorFactory: createManyAccessor
  });
};

var withOne = function withOne(name, opts) {
  var _parse = parse(opts),
      hasMany = _parse.hasMany,
      relatedName = _parse.relatedName,
      relatedModel = _parse.model;

  return hasMany ? createOneToManyRelation({ name: name, relatedName: relatedName, relatedModel: relatedModel }) : createOneToOneRelation({ name: name, relatedName: relatedName, relatedModel: relatedModel });
};

var withMany = function withMany(name, opts) {
  var _parse2 = parse(opts),
      hasMany = _parse2.hasMany,
      relatedName = _parse2.relatedName,
      relatedModel = _parse2.model;

  return hasMany ? createManyToManyRelation({ name: name, relatedName: relatedName, relatedModel: relatedModel }) : createManyToOneRelation({ name: name, relatedName: relatedName, relatedModel: relatedModel });
};

module.exports = {
  withOne: withOne,
  withMany: withMany
};
