'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ulid = require('ulid');
var assert = require('assert');
var Immutable = require('immutable');

var ENTRY_DOES_NOT_EXIST = function ENTRY_DOES_NOT_EXIST(id) {
  return ('\n  Entry with id \'' + id + '\' does not exist.\n').trim();
};

var SYNC_NO_MAP = '\n  Trying to sync values with invalid type.\n'.trim();

var Transaction = function () {
  function Transaction(_ref) {
    var values = _ref.values,
        setup = _ref.setup;

    _classCallCheck(this, Transaction);

    this.values = values;
    this.keyName = setup.get('keyName');
  }

  _createClass(Transaction, [{
    key: 'syncNew',
    value: function syncNew(values) {
      var id = ulid();
      var newValues = values.set(this.keyName, id);

      assert.ok(newValues instanceof Immutable.Map, SYNC_NO_MAP);

      this.values = this.values.set(id, newValues);

      return newValues;
    }
  }, {
    key: 'syncExisting',
    value: function syncExisting(values) {
      var id = values.get(this.keyName);

      assert.ok(this.values.has(id), ENTRY_DOES_NOT_EXIST(id));
      assert.ok(values instanceof Immutable.Map, SYNC_NO_MAP);

      if (values !== this.values.get(id)) {
        this.values = this.values.set(id, values);
      }

      return values;
    }
  }, {
    key: 'sync',
    value: function sync(values) {
      return values.has(this.keyName) ? this.syncExisting(values) : this.syncNew(values);
    }
  }, {
    key: 'unsync',
    value: function unsync(values) {
      this.values = this.values.remove(values.get(this.keyName));
      return values.remove(this.keyName);
    }
  }]);

  return Transaction;
}();

module.exports = {
  Transaction: Transaction
};
